@@ -60,7 +60,13 @@ public class RJavaClassLoader extends URLClassLoader {
 	    so = new UnixFile(rJavaLibPath+"/rJava.dll");
 	if (so.exists())
 	    libMap.put("rJava", so);
+	
 	UnixFile jri = new UnixFile(path+"/jri/libjri.so");
+	String rarch = System.getProperty("r.arch");
+	if (rarch != null && rarch.length()>0) {
+	    UnixFile af = new UnixFile(path+"/jri"+rarch+"/libjri.so");
+	    if (af.exists()) jri=af;
+	}
 	if (!jri.exists())
 	    jri = new UnixFile(path+"/jri/libjri.jnilib");
 	if (!jri.exists())
